<?php

namespace App\Service\SnipParser\Stages;

use App\Repository\SnipRepository;
use App\Security\Voter\SnipVoter;
use App\Service\SnipParser\Pipeline;
use App\Service\SnipServiceFactory;
use League\Pipeline\StageInterface;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

class IncludeReferenceStage implements StageInterface
{
    public function __construct(
        #[Autowire(lazy: true)] private readonly Security           $security,
        #[Autowire(lazy: true)] private readonly SnipRepository     $snipRepository,
        #[Autowire(lazy: true)] private readonly SnipServiceFactory $snipServiceFactory,
        #[Autowire(lazy: true)] private readonly Pipeline           $pipeline,
    ) {}

    public function __invoke(mixed $payload): string
    {
        return $this->replaceReferences($payload);
    }

    private function replaceReferences(mixed $payload): string
    {
        // replaces all references (#n) to other snips with links
        $pattern = '/\{\{(\d+)\}\}/';

        return preg_replace_callback($pattern, function ($matches) {
            $snip = $this->snipRepository->find($matches[1]);
            if ($snip === null) {
                return sprintf('<span title="not found">%s</span>', $matches[0]);
            }
            if (!$this->security->isGranted(SnipVoter::VIEW, $snip)) {
                return sprintf('<span title="access denied">%s</span>', $matches[0]);
            }

            return $this->pipeline->parse($this->snipServiceFactory->create($snip)->get());
        }, $payload);
    }
}